def normalize(df, old_column, new_column):
	df[new_column] = df[old_column] / (df["ny"] * df["nx"])
    
def print_and_return_fit(list_of_quantities, dataframe, function, format_value=">7.4f", format_uncertainty="f", _print=True):
    """Use `curve_fit` to fit each quantity in `list_of_quantity` wrt to `dataframe.index`. Print (selectable) and return the result."""
    import numpy as np
    from scipy.optimize import curve_fit 
    _fit_parameters = {}
    _fit_covariance = {}
    _quantity_padding = np.max([len(_str) for _str in list_of_quantities])
    for quantity in list_of_quantities:
        _fit_parameters[quantity], _fit_covariance[quantity] = curve_fit(function, dataframe.index, dataframe[quantity])
        if (_print):
            print("Counter {:>{_quantity_padding}} is proportional to the grid points (nx*ny) by a factor of {:{format_value}} (± {:{format_uncertainty}})".format(
                quantity, 
                _fit_parameters[quantity][0], 
                np.sqrt(np.diag(_fit_covariance[quantity]))[0],
                _quantity_padding=_quantity_padding,
                format_value=format_value,
                format_uncertainty=format_uncertainty
        ))
    return (_fit_parameters, _fit_covariance)