import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
sns.set()
plt.rcParams['figure.figsize'] = [14, 6]

import common

def task1(input="poisson2d.ins_cyc.bin.csv"):
	df = pd.read_csv(input, skiprows=range(2, 50000, 2))  # Read in the CSV file from the bench run; parse with Pandas
	common.normalize(df, "PM_INST_CMPL (min)", "Instructions / Loop Iteration")  # Normalize to each iteration
	common.normalize(df, "PM_RUN_CYC (min)", "Cycles / Loop Iteration")
	fig, (ax1, ax2) = plt.subplots(nrows=2, sharex=True)
	df.set_index("nx")["Cycles / Loop Iteration"].plot(ax=ax1, legend=True);
	df.set_index("nx")["Instructions / Loop Iteration"].plot(ax=ax2, legend=True);
	fig.savefig("plot-task1.pdf")

def task2a(input="poisson2d.ld_st.bin.csv"):
	df_ldst = pd.read_csv(input, skiprows=range(2, 50000, 2))
	common.normalize(df_ldst, "PM_LD_CMPL (min)", "Loads / Loop Iteration")
	common.normalize(df_ldst, "PM_ST_CMPL (min)", "Stores / Loop Iteration")
	fig, (ax1, ax2) = plt.subplots(nrows=2, sharex=True)
	df_ldst.set_index("nx")["Loads / Loop Iteration"].plot(ax=ax1, legend=True);
	df_ldst.set_index("nx")["Stores / Loop Iteration"].plot(ax=ax2, legend=True);
	fig.savefig("plot-task2a.pdf")


def task2b(input1="poisson2d.vld.bin.csv", input2="poisson2d.vst.bin.csv", input3="poisson2d.ld_st.bin.csv", bytes=False):
	df_vld = pd.read_csv(input1, skiprows=range(2, 50000, 2))
	df_vst = pd.read_csv(input2, skiprows=range(2, 50000, 2))
	df_vldvst = pd.concat([df_vld.set_index("nx"), df_vst.set_index("nx")[['PM_VECTOR_ST_CMPL (total)', 'PM_VECTOR_ST_CMPL (min)', ' PM_VECTOR_ST_CMPL (max)']]], axis=1).reset_index()
	common.normalize(df_vldvst, "PM_VECTOR_LD_CMPL (min)", "Vector Loads / Loop Iteration")
	common.normalize(df_vldvst, "PM_VECTOR_ST_CMPL (min)", "Vector Stores / Loop Iteration")
	if bytes is False:
		fig, (ax1, ax2) = plt.subplots(nrows=2, sharex=True)
		df_vldvst.set_index("nx")["Vector Loads / Loop Iteration"].plot(ax=ax1, legend=True);
		df_vldvst.set_index("nx")["Vector Stores / Loop Iteration"].plot(ax=ax2, legend=True);
		fig.savefig("plot-task2b.pdf")
	else:
		df_ldst = pd.read_csv(input3, skiprows=range(2, 50000, 2))
		common.normalize(df_ldst, "PM_LD_CMPL (min)", "Loads / Loop Iteration")
		common.normalize(df_ldst, "PM_ST_CMPL (min)", "Stores / Loop Iteration")
		df_byte = pd.DataFrame()
		df_byte["Loads / Loop Iteration"] = (df_vldvst.set_index("nx")["Vector Loads / Loop Iteration"] + df_ldst.set_index("nx")["Loads / Loop Iteration"])*8
		df_byte["Stores / Loop Iteration"] = (df_vldvst.set_index("nx")["Vector Stores / Loop Iteration"] + df_ldst.set_index("nx")["Stores / Loop Iteration"])*8
		fig, ax = plt.subplots()
		ax = df_byte.plot(ax=ax)
		ax.set_ylabel("Bytes / Loop Iteration");
		fig.savefig("plot-task2b-2.pdf")

def task2c(input1="poisson2d.vld.bin.csv", input2="poisson2d.vst.bin.csv", input3="poisson2d.ld_st.bin.csv", input4="poisson2d.ins_cyc.bin.csv"):
	df = pd.read_csv(input4, skiprows=range(2, 50000, 2))
	common.normalize(df, "PM_INST_CMPL (min)", "Instructions / Loop Iteration")
	common.normalize(df, "PM_RUN_CYC (min)", "Cycles / Loop Iteration")
	df_vld = pd.read_csv(input1, skiprows=range(2, 50000, 2))
	df_vst = pd.read_csv(input2, skiprows=range(2, 50000, 2))
	df_vldvst = pd.concat([df_vld.set_index("nx"), df_vst.set_index("nx")[['PM_VECTOR_ST_CMPL (total)', 'PM_VECTOR_ST_CMPL (min)', ' PM_VECTOR_ST_CMPL (max)']]], axis=1).reset_index()
	common.normalize(df_vldvst, "PM_VECTOR_LD_CMPL (min)", "Vector Loads / Loop Iteration")
	common.normalize(df_vldvst, "PM_VECTOR_ST_CMPL (min)", "Vector Stores / Loop Iteration")
	df_ldst = pd.read_csv(input3, skiprows=range(2, 50000, 2))
	common.normalize(df_ldst, "PM_LD_CMPL (min)", "Loads / Loop Iteration")
	common.normalize(df_ldst, "PM_ST_CMPL (min)", "Stores / Loop Iteration")
	df_byte = pd.DataFrame()
	df_byte["Loads / Loop Iteration"] = (df_vldvst.set_index("nx")["Vector Loads / Loop Iteration"] + df_ldst.set_index("nx")["Loads / Loop Iteration"])*8
	df_byte["Stores / Loop Iteration"] = (df_vldvst.set_index("nx")["Vector Stores / Loop Iteration"] + df_ldst.set_index("nx")["Stores / Loop Iteration"])*8
	df_bandwidth = pd.DataFrame()
	df_bandwidth["Bandwidth / Byte/Cycle"] = (df_byte["Loads / Loop Iteration"] + df_byte["Stores / Loop Iteration"]) / df.set_index("nx")["Cycles / Loop Iteration"]
	fig, (ax1, ax2) = plt.subplots(nrows=2, sharex=True)
	for ax in [ax1, ax2]:
	    df_bandwidth["Bandwidth / Byte/Cycle"].plot(ax=ax, legend=True, label="Jacobi Bandwidth")
	    ax.set_ylabel("Byte/Cycle")
	ax2.axhline(2*16, color=sns.color_palette()[1], label="L1 Bandwidth");
	ax2.legend();
	fig.savefig("plot-task2c.pdf")

def task4(input1="poisson2d.vld.bin.csv", input2="poisson2d.vst.bin.csv", input3="poisson2d.ld_st.bin.csv", input4="poisson2d.ins_cyc.bin.csv", input5="poisson2d.sflop.bin.csv", input6="poisson2d.vflop.bin.csv", ai=False):
	df_sflop = pd.read_csv(input5, skiprows=range(2, 50000, 2))
	df_vflop = pd.read_csv(input6, skiprows=range(2, 50000, 2))
	df_flop = pd.concat([df_sflop.set_index("nx"), df_vflop.set_index("nx")[['PM_VECTOR_FLOP_CMPL (total)', 'PM_VECTOR_FLOP_CMPL (min)', ' PM_VECTOR_FLOP_CMPL (max)']]], axis=1).reset_index()
	common.normalize(df_flop, "PM_SCALAR_FLOP_CMPL (min)", "Scalar FlOps / Loop Iteration")
	common.normalize(df_flop, "PM_VECTOR_FLOP_CMPL (min)", "Vector Instructions / Loop Iteration")
	df_flop["Vector FlOps / Loop Iteration"] = df_flop["Vector Instructions / Loop Iteration"] * 2
	if ai is False:
		fig, ax = plt.subplots()
		df_flop.set_index("nx")[["Scalar FlOps / Loop Iteration", "Vector FlOps / Loop Iteration"]].plot(ax=ax);
		fig.savefig("plot-task4.pdf")
	else:
		df_vld = pd.read_csv(input1, skiprows=range(2, 50000, 2))
		df_vst = pd.read_csv(input2, skiprows=range(2, 50000, 2))
		df_vldvst = pd.concat([df_vld.set_index("nx"), df_vst.set_index("nx")[['PM_VECTOR_ST_CMPL (total)', 'PM_VECTOR_ST_CMPL (min)', ' PM_VECTOR_ST_CMPL (max)']]], axis=1).reset_index()
		common.normalize(df_vldvst, "PM_VECTOR_LD_CMPL (min)", "Vector Loads / Loop Iteration")
		common.normalize(df_vldvst, "PM_VECTOR_ST_CMPL (min)", "Vector Stores / Loop Iteration")
		df_ldst = pd.read_csv(input3, skiprows=range(2, 50000, 2))
		common.normalize(df_ldst, "PM_LD_CMPL (min)", "Loads / Loop Iteration")
		common.normalize(df_ldst, "PM_ST_CMPL (min)", "Stores / Loop Iteration")
		df_byte = pd.DataFrame()
		df_byte["Loads / Loop Iteration"] = (df_vldvst.set_index("nx")["Vector Loads / Loop Iteration"] + df_ldst.set_index("nx")["Loads / Loop Iteration"])*8
		df_byte["Stores / Loop Iteration"] = (df_vldvst.set_index("nx")["Vector Stores / Loop Iteration"] + df_ldst.set_index("nx")["Stores / Loop Iteration"])*8
		I_flop_scalar = df_flop.set_index("nx")["Scalar FlOps / Loop Iteration"]
		I_flop_vector = df_flop.set_index("nx")["Vector FlOps / Loop Iteration"]
		I_mem_load    = df_byte["Loads / Loop Iteration"]
		I_mem_store   = df_byte["Stores / Loop Iteration"]
		df_ai = pd.DataFrame()
		df_ai["Arithmetic Intensity"] = (I_flop_scalar + I_flop_vector) / (I_mem_load + I_mem_store)
		fig, ax = plt.subplots()
		df_ai.plot(ax=ax);
		ax.set_ylabel("Byte/FlOp");
		fig.savefig("plot-task4-2.pdf")
